@@ -17,6 +17,7 @@ from tempfile import TemporaryFile
 import six
 
 import pytest
+from _pytest.compat import _PY3
 from _pytest.compat import CaptureIO
 
 patchsysdict = {0: "stdin", 1: "stdout", 2: "stderr"}
@@ -412,6 +413,10 @@ class EncodedFile(object):
     def write(self, obj):
         if isinstance(obj, six.text_type):
             obj = obj.encode(self.encoding, "replace")
+        elif _PY3:
+            raise TypeError(
+                "write() argument must be str, not {}".format(type(obj).__name__)
+            )
         self.buffer.write(obj)
 
     def writelines(self, linelist):
@@ -773,9 +778,9 @@ def _py36_windowsconsoleio_workaround(stream):
             f.line_buffering,
         )
 
-    sys.__stdin__ = sys.stdin = _reopen_stdio(sys.stdin, "rb")
-    sys.__stdout__ = sys.stdout = _reopen_stdio(sys.stdout, "wb")
-    sys.__stderr__ = sys.stderr = _reopen_stdio(sys.stderr, "wb")
+    sys.stdin = _reopen_stdio(sys.stdin, "rb")
+    sys.stdout = _reopen_stdio(sys.stdout, "wb")
+    sys.stderr = _reopen_stdio(sys.stderr, "wb")
 
 
 def _attempt_to_close_capture_file(f):
