@@ -1,15 +1,21 @@
 """ discover and run doctests in modules and test files."""
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
-import traceback
-import sys
+import inspect
 import platform
+import sys
+import traceback
+from contextlib import contextmanager
 
 import pytest
-from _pytest._code.code import ExceptionInfo, ReprFileLocation, TerminalRepr
+from _pytest._code.code import ExceptionInfo
+from _pytest._code.code import ReprFileLocation
+from _pytest._code.code import TerminalRepr
+from _pytest.compat import safe_getattr
 from _pytest.fixtures import FixtureRequest
 
-
 DOCTEST_REPORT_CHOICE_NONE = "none"
 DOCTEST_REPORT_CHOICE_CDIFF = "cdiff"
 DOCTEST_REPORT_CHOICE_NDIFF = "ndiff"
@@ -203,7 +209,8 @@ class DoctestItem(pytest.Item):
             return
         capman = self.config.pluginmanager.getplugin("capturemanager")
         if capman:
-            out, err = capman.suspend_global_capture(in_=True)
+            capman.suspend_global_capture(in_=True)
+            out, err = capman.read_global_capture()
             sys.stdout.write(out)
             sys.stderr.write(err)
 
@@ -341,10 +348,61 @@ def _check_all_skipped(test):
         pytest.skip("all tests skipped by +SKIP option")
 
 
+def _is_mocked(obj):
+    """
+    returns if a object is possibly a mock object by checking the existence of a highly improbable attribute
+    """
+    return (
+        safe_getattr(obj, "pytest_mock_example_attribute_that_shouldnt_exist", None)
+        is not None
+    )
+
+
+@contextmanager
+def _patch_unwrap_mock_aware():
+    """
+    contextmanager which replaces ``inspect.unwrap`` with a version
+    that's aware of mock objects and doesn't recurse on them
+    """
+    real_unwrap = getattr(inspect, "unwrap", None)
+    if real_unwrap is None:
+        yield
+    else:
+
+        def _mock_aware_unwrap(obj, stop=None):
+            if stop is None:
+                return real_unwrap(obj, stop=_is_mocked)
+            else:
+                return real_unwrap(obj, stop=lambda obj: _is_mocked(obj) or stop(obj))
+
+        inspect.unwrap = _mock_aware_unwrap
+        try:
+            yield
+        finally:
+            inspect.unwrap = real_unwrap
+
+
 class DoctestModule(pytest.Module):
     def collect(self):
         import doctest
 
+        class MockAwareDocTestFinder(doctest.DocTestFinder):
+            """
+            a hackish doctest finder that overrides stdlib internals to fix a stdlib bug
+
+            https://github.com/pytest-dev/pytest/issues/3456
+            https://bugs.python.org/issue25532
+            """
+
+            def _find(self, tests, obj, name, module, source_lines, globs, seen):
+                if _is_mocked(obj):
+                    return
+                with _patch_unwrap_mock_aware():
+
+                    doctest.DocTestFinder._find(
+                        self, tests, obj, name, module, source_lines, globs, seen
+                    )
+
         if self.fspath.basename == "conftest.py":
             module = self.config.pluginmanager._importconftest(self.fspath)
         else:
@@ -356,7 +414,7 @@ class DoctestModule(pytest.Module):
                 else:
                     raise
         # uses internal doctest module parsing mechanism
-        finder = doctest.DocTestFinder()
+        finder = MockAwareDocTestFinder()
         optionflags = get_optionflags(self)
         runner = _get_runner(
             verbose=0,
